// Function to calculate a random delay in milliseconds within a given range
const getRandomDelay = (minMinutes, maxMinutes) =>
  (minMinutes + Math.random() * (maxMinutes - minMinutes)) * 60 * 1000;

// Function to check if an alarm is in an acceptable range
const isAlarmInAcceptableRange = (existingAlarm, nextTriggerTime) =>
  existingAlarm &&
  Math.abs(existingAlarm.scheduledTime - nextTriggerTime) <= 20 * 60 * 1000; // 20 minutes in milliseconds
// Function to calculate the next trigger time
function calculateNextTriggerTime(timestamp, waitPeriod) {
  const baseDelay = waitPeriod * 60 * 60 * 1000; // Convert hours to milliseconds
  const randomDelay = getRandomDelay(6, 17); // Random delay between 6 and 17 minutes
  const lastTimestamp = new Date(timestamp).getTime();

  if (isNaN(lastTimestamp)) {
    console.error("Invalid timestamp received:", timestamp);
    return null; // Or handle the error as appropriate
  }

  const nextTriggerTime = lastTimestamp + baseDelay + randomDelay;
  // console.log(
  // //   "Next trigger time calculated:",
  // //   new Date(nextTriggerTime).toLocaleString()
  // // );
  return nextTriggerTime;
}
// Function to determine the delay for retry alarms based on fail count
const checkAndSetAlarmForAccount = async (accountId) => {
  const specificKeys = [
    `timestamp_for_${accountId}`,
    `waitPeriod_for_${accountId}`,
    `failCount_for_${accountId}`,
    `manualAlarmSet_${accountId}`,
    `isActive_${accountId}`,
    `paused_${accountId}`,
  ];

  try {
    const accountData = await getStorageData(specificKeys);

    const {
      [`timestamp_for_${accountId}`]: timestamp,
      [`waitPeriod_for_${accountId}`]: waitPeriod,
      [`failCount_for_${accountId}`]: failCount = 0,
      [`isActive_${accountId}`]: isActive,
      [`manualAlarmSet_${accountId}`]: manualAlarmSet,
      [`paused_${accountId}`]: paused,
    } = accountData;

    if (!timestamp || !isActive || manualAlarmSet || paused) {
      console.warn(`Skipping alarm setup for account ${accountId}.`);
      return;
    }

    const nextTriggerTime = calculateNextTriggerTime(timestamp, waitPeriod);
    if (!nextTriggerTime) {
      console.error(`Failed to calculate next trigger time for ${accountId}.`);
      return;
    }

    const existingAlarm = await new Promise((resolve) =>
      chrome.alarms.get(`collectAlarm_${accountId}`, resolve)
    );

    const alarmCase = determineAlarmCase(
      existingAlarm,
      failCount,
      nextTriggerTime
    );

    await handleAlarmCase(
      alarmCase,
      accountId,
      nextTriggerTime,
      failCount,
      existingAlarm
    );
  } catch (error) {
    console.error(`Error in checkAndSetAlarmForAccount(${accountId}):`, error);
  }
};

const setAlarm = async (accountId, nextTriggerTime, failCount, isRetry) => {
  const currentTime = Date.now();
  const delay = isRetry
    ? getRetryDelay(failCount)
    : Math.max(nextTriggerTime - currentTime, 0);

  const alarmName = `collectAlarm_${accountId}`;
  try {
    chrome.alarms.create(alarmName, { when: currentTime + delay });
    console.log(
      `${
        isRetry ? "Retry" : "Regular"
      } alarm set for ${accountId} at ${new Date(
        currentTime + delay
      ).toLocaleString()}. Fail count: ${failCount}`
    );
  } catch (error) {
    console.error(`Failed to create alarm for ${accountId}:`, error);
  }
};

const determineAlarmCase = (existingAlarm, failCount, nextTriggerTime) => {
  if (!existingAlarm) {
    return failCount > 0 ? "NEW_RETRY" : "NEW_REGULAR";
  }

  if (failCount > 0) {
    return "UPDATE_RETRY";
  }

  return isAlarmInAcceptableRange(existingAlarm, nextTriggerTime)
    ? "KEEP_EXISTING"
    : "UPDATE_REGULAR";
};
const handleAlarmCase = async (
  alarmCase,
  accountId,
  nextTriggerTime,
  failCount
) => {
  switch (alarmCase) {
    case "NEW_RETRY":
      console.log(`Creating retry alarm for account ${accountId}.`);
      await setAlarm(accountId, nextTriggerTime, failCount, true);
      break;

    case "NEW_REGULAR":
      console.log(`Creating regular alarm for account ${accountId}.`);
      await setAlarm(accountId, nextTriggerTime, 0, false);
      break;

    case "UPDATE_RETRY":
      console.log(`Updating retry alarm for account ${accountId}.`);
      await setAlarm(accountId, nextTriggerTime, failCount, true);
      break;

    case "UPDATE_REGULAR":
      console.log(
        `Updating regular alarm for account ${accountId}. Next trigger time: ${new Date(
          nextTriggerTime
        ).toLocaleString()}`
      );
      await setAlarm(accountId, nextTriggerTime, 0, false);
      break;

    case "KEEP_EXISTING":
      console.log(`No update needed for alarm ${accountId}.`);
      break;

    default:
      console.error(`Unknown alarm case for account ${accountId}.`);
  }
};

const getRetryDelay = (failCount) => {
  const retryDelays = [2, 15, 45, 4 * 60]; // in minutes
  const delayMinutes = retryDelays[failCount - 1] || 720; // Default to 12 hours if failCount is out of range
  return delayMinutes * 60 * 1000; // Convert to milliseconds
};

// Alarm listener with separated handlers for different alarm types
chrome.alarms.onAlarm.addListener(async (alarm) => {
  if (alarm.name.startsWith("collectAlarm_")) {
    await handleCollectionAlarm(alarm);
  } else if (alarm.name === initializationAlarmName) {
    initializeAlarmManager();
  } else if (alarm.name === "dailyBalanceLogging") {
    logAllAccountBalances();
  }
});
// Dedicated handler for collection alarms
async function handleCollectionAlarm(alarm) {
  const accountId = alarm.name.replace("collectAlarm_", "");
  // const timeSinceLastActivity = Date.now() - lastUserActivity;
  // if (timeSinceLastActivity < INACTIVITY_THRESHOLD) {
  //   console.log(
  //     `User was active ${Math.round(
  //       timeSinceLastActivity / 1000 / 60
  //     )} minutes ago, skipping check`
  //   );
  //   return;
  // }
  try {
    const { storedAccounts = {} } = await getStorageData("storedAccounts");
    const account = storedAccounts[accountId];

    if (!isValidAccount(account)) {
      throw new Error(`Invalid or missing account data for ID: ${accountId}`);
    }

    await performCollection(accountId, account.name, account.URL);
  } catch (error) {
    console.error(`Collection failed for ${accountId}:`, error.message);
  }
}
// Listener for the initialization alarm
chrome.alarms.onAlarm.addListener((alarm) => {
  if (alarm.name === initializationAlarmName) {
    console.log(
      "Initialization alarm triggered. Reinitializing alarm manager."
    );
    initializeAlarmManager();
  } else if (alarm.name === "dailyBalanceLogging") {
    logAllAccountBalances();
  }
});
//
// Set up the initialization alarm - runs every xx hours
const initializationAlarmName = "initializeAlarmManager";
let lastUserActivity = Date.now();

chrome.alarms.create(initializationAlarmName, { periodInMinutes: 1 * 60 });

chrome.runtime.onStartup.addListener(() => {
  console.log("Browser started. Initializing alarms...");
  initializeAlarmManager();
  setDailyBalanceLoggingAlarm();
});

// Daily balance logging alarm setup
function setDailyBalanceLoggingAlarm() {
  const now = new Date();
  const midnightPST = new Date();
  midnightPST.setUTCHours(8, 0, 0, 0); // 12:00 AM PST is 08:00 AM UTC

  if (now.getUTCHours() >= 8) {
    midnightPST.setUTCDate(now.getUTCDate() + 1);
  }

  const delayInMinutes = (midnightPST.getTime() - now.getTime()) / 1000 / 60;
  chrome.alarms.create("dailyBalanceLogging", {
    delayInMinutes,
    periodInMinutes: 1440,
  }); // Repeat every 24 hours
  console.log(
    `Daily balance logging alarm set for ${midnightPST.toLocaleString()} PST.`
  );
}

// Function to log all account balances
function logAllAccountBalances() {
  chrome.storage.local.get("storedAccounts", (result) => {
    const accounts = result.storedAccounts || {};

    Object.keys(accounts).forEach((accountId) => {
      const balance = accounts[accountId]?.currentVal || 0;
      const balanceEntry = { date: new Date().toISOString(), balance };

      chrome.storage.local.get([`balanceHistory_${accountId}`], (result) => {
        const history = result[`balanceHistory_${accountId}`] || [];
        history.push(balanceEntry);

        chrome.storage.local.set(
          { [`balanceHistory_${accountId}`]: history },
          () => {
            console.log(
              `Logged balance for account ${accountId}:`,
              balanceEntry
            );
          }
        );
      });
    });
  });
}
//

// Add this to your initializeAlarmManager function
function initializeAlarmManager() {
  setDailyBalanceLoggingAlarm();
}

async function handleFlaggedAccountsCheck() {
  try {
    const result = await getStorageData("storedAccounts");
    const accounts = result.storedAccounts || {};

    // Get accounts and their fail counts in parallel
    const accountPromises = Object.entries(accounts).map(
      async ([accountId, account]) => {
        const accountData = await getStorageData([
          `failCount_for_${accountId}`,
          `isActive_${accountId}`, // Fixed: was using account.id
        ]);

        return {
          accountId,
          account,
          failCount: accountData[`failCount_for_${accountId}`] || 0,
          isActive: accountData[`isActive_${accountId}`] || false,
        };
      }
    );

    // Wait for all checks to complete
    const accountsWithData = await Promise.all(accountPromises);

    // Filter for active accounts with failCount > 0
    const flaggedAccounts = accountsWithData
      .filter(({ failCount, isActive }) => isActive && failCount > 0)
      .map(({ accountId, account }) => ({
        accountId,
        accountName: account.name,
        URL: account.URL,
      }));

    console.log(
      `Found ${flaggedAccounts.length} active flagged accounts to process`
    );

    // Process each flagged active account
    flaggedAccounts.forEach((account) => {
      performCollection(
        account.accountId,
        account.accountName,
        account.URL,
        true
      );
    });
  } catch (error) {
    console.error("Error in flagged accounts check:", error);
  }
}
// Helper function to validate account data
function isValidAccount(account) {
  return account && account.name && account.URL;
}
const saveSchedule = () => {
  chrome.storage.local.set({ resolveTimes: selectedTimes }, () => {
    alert("Resolve times saved!");
    setShowSchedulePopup(false);

    // Get all alarms and remove only those related to the resolve schedule
    chrome.alarms.getAll((alarms) => {
      alarms.forEach((alarm) => {
        if (alarm.name.startsWith("resolve-")) {
          chrome.alarms.clear(alarm.name);
        }
      });

      // Set new alarms for each selected time
      selectedTimes.forEach((time) => {
        setDailyAlarm(time);
      });
    });
  });
};

const setDailyAlarm = (time) => {
  // Parse the "HH:MM" format
  const [hours, minutes] = time.split(":").map(Number);

  const now = new Date();
  const alarmTime = new Date(
    now.getFullYear(),
    now.getMonth(),
    now.getDate(),
    hours,
    minutes,
    0,
    0
  );

  // If the chosen time has already passed today, schedule it for tomorrow
  if (alarmTime.getTime() <= now.getTime()) {
    alarmTime.setDate(alarmTime.getDate() + 1);
  }
  // console.log(
  //   `Creating daily alarm "resolve-${time}" at ${alarmTime.toLocaleString()}`
  // );
  chrome.alarms.create(`resolve-${time}`, {
    when: alarmTime.getTime(),
    periodInMinutes: 24 * 60, // 24 hours in minutes for daily recurrence
  });
};
chrome.alarms.onAlarm.addListener((alarm) => {
  if (alarm.name.startsWith("resolve-")) {
    console.log(`Triggering scheduled resolve at ${alarm.name.split("-")[1]}`);

    // Directly call the flagged accounts check at the scheduled time
    handleFlaggedAccountsCheck().catch((error) => {
      console.error("Error running flagged accounts check:", error);
    });
  }
});
