// Description: Helper functions for the background script

// Function to send a message to a tab and return the response
async function sendMessageToTab(tabId, data) {
  console.log("Action", data.action);
  return new Promise((resolve) => {
    chrome.tabs.sendMessage(tabId, data, (response) => {
      if (chrome.runtime.lastError) {
        console.error(
          "Error sending message:",
          chrome.runtime.lastError.message
        );
        resolve(null);
      } else if (response?.status === "success") {
        resolve(response.value || response);
      } else {
        console.error(`${data.action}: Unexpected response`, response);
        resolve(null);
      }
    });
  });
}

const fetchFindVal = (tabId, params) => sendMessageToTab(tabId, params);

const fetchClickButton = (tabId, params) => sendMessageToTab(tabId, params);

const triggerSimulateTyping = async (tabId, params) => {
  const { slug } = params;
  // console.log("triggerSimulateTyping", params);
  try {
    const keys = await getStorageData([`UserKeys_${slug}`, `PassKeys_${slug}`]);
    const userText = keys[`UserKeys_${slug}`];
    const passText = keys[`PassKeys_${slug}`];

    if (userText && passText) {
      // Merge the userText and passText into the params object
      return sendMessageToTab(tabId, {
        ...params,
        userText,
        passText,
        action: "simulateTyping", // Ensure the action is included
      });
    } else {
      throw new Error("User or Pass key not found");
    }
  } catch (error) {
    console.error("Typing simulation failed", error);
    throw error;
  }
};

async function triggerNavigation(tabId, params) {
  const response = await sendMessageToTab(tabId, params);
  if (response) console.log("Redirected to URL:", params.URL);
}
// Function to check if the account has keys stored
function hasKeys(accountId, callback) {
  chrome.storage.local.get("storedAccounts", (result) => {
    const accountExists = result.storedAccounts?.[accountId]?.hasKeys ?? false;
    callback(accountExists);
  });
}

// Function to process the next collection in the queue if there's capacity
async function switchTab(action, originalTabId) {
  const authMethod = action.authMethod;
  console.log(`Switching to ${authMethod} OAuth tab...`);

  let oauthUrlPattern = "";

  if (authMethod === "google") {
    oauthUrlPattern = "https://accounts.google.com/o/oauth2/v2/*";
  } else if (authMethod === "facebook") {
    oauthUrlPattern = "https://www.facebook.com/dialog/oauth*";
  }

  // Locate the OAuth tab
  const [oauthTab] = await new Promise((resolve) =>
    chrome.tabs.query({ url: oauthUrlPattern }, resolve)
  );

  if (oauthTab) {
    console.log(`Found ${authMethod} OAuth tab with ID:`, oauthTab.id);

    // Add a listener to detect when the OAuth tab is closed
    chrome.tabs.onRemoved.addListener(function closedTabListener(tabId) {
      if (tabId === oauthTab.id) {
        console.log(
          `OAuth tab with ID ${tabId} closed. Switching back to original tab.`
        );

        // Switch back to the original tab
        chrome.tabs.update(originalTabId, { active: true });

        // Remove this listener to avoid memory leaks
        chrome.tabs.onRemoved.removeListener(closedTabListener);
      }
    });

    return oauthTab.id; // Return the new tabId for subsequent actions
  } else {
    throw new Error(`Failed to find ${authMethod} OAuth tab.`);
  }
}

//Utility function to get data from storage
function getStorageData(keys) {
  return new Promise((resolve, reject) => {
    chrome.storage.local.get(keys, (result) => {
      if (chrome.runtime.lastError) {
        reject(chrome.runtime.lastError.message);
      } else {
        resolve(result);
      }
    });
  });
}
//Helper function check if internet connection is available
async function checkInternetConnection() {
  try {
    const response = await fetch(
      "https://jsonplaceholder.typicode.com/posts/1",
      {
        method: "HEAD",
        cache: "no-cache",
      }
    );
    return response.ok;
  } catch (error) {
    console.warn("Internet connection check failed:", error);
    return false;
  }
}

async function countQualifyingAccounts() {
  try {
    const items = await getStorageData(null); // Get all items
    let count = 0;
    const accountIds = new Set();

    // Collect all account IDs
    for (let key in items) {
      if (key.startsWith("isActive_")) {
        accountIds.add(key.split("_")[1]);
      }
    }

    // Check criteria for each account
    for (let id of accountIds) {
      const accountData = await getStorageData([
        `isActive_${id}`,
        `failCount_for_${id}`,
        `paused_${id}`,
      ]);

      const isActive = accountData[`isActive_${id}`];
      const failCount = accountData[`failCount_for_${id}`] || 0;
      const isPaused = accountData[`paused_${id}`];

      if (isActive && failCount > 0 && !isPaused) {
        count++;
      }
    }

    return count;
  } catch (error) {
    console.error("Error counting qualifying accounts:", error);
    return 0;
  }
}

async function updateBadge() {
  try {
    const count = await countQualifyingAccounts();

    if (count === 0) {
      // Clear the badge text completely
      chrome.action.setBadgeText({ text: "" });
      // Optional: Also clear the background color
      chrome.action.setBadgeBackgroundColor({ color: [0, 0, 0, 0] });
    } else {
      // Show count and red background for non-zero values
      chrome.action.setBadgeText({ text: count.toString() });
      chrome.action.setBadgeBackgroundColor({ color: "#FF8080" }); // Red
    }
  } catch (error) {
    console.error("Error updating badge:", error);
  }
}

// Consolidated event listeners
function setupListeners() {
  updateBadge();
  chrome.storage.onChanged.addListener((changes, namespace) => {
    if (namespace === "local") updateBadge();
  });

  chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    switch (message.action) {
      case "executeFunction": {
        const { accountId, accountName, URL, active } = message;
        console.log("Processing account:", {
          accountId,
          accountName,
          URL,
          active,
        });

        try {
          performCollection(accountId, accountName, URL, active);
          console.log("Successfully processed account:", accountId);
          sendResponse({ success: true });
        } catch (error) {
          console.error("Error processing account:", accountId, error);
          sendResponse({ success: false, error: error.message });
        }
        break;
      }

      case "setAlarms": {
        const { times } = message;
        // console.log("Received times to set alarms:", times);

        // Clear only resolve-related alarms before setting new ones
        chrome.alarms.getAll((alarms) => {
          alarms.forEach((alarm) => {
            if (alarm.name.startsWith("resolve-")) {
              chrome.alarms.clear(alarm.name);
            }
          });

          // Set new daily alarms for each time
          times.forEach((time) => setDailyAlarm(time));

          // Respond after the alarms are scheduled
          sendResponse({ status: "alarms_set" });
        });

        return true; // Keep the message channel open for asynchronous response
      }

      default:
        // Optionally handle unknown actions here
        // console.warn("Unknown message action:", message.action);
        // sendResponse({ success: false, error: "Unknown action" });
        break;
    }

    return true; // Keep message channel open for async response if needed
  });
}

setupListeners();
